﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Functional
{
    public class MatrixMaps
    {
        public static T3[,] Map<T1, T2, T3>(T1[,] inMatrix, T2 inScalar, Func<T1, T2, T3> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix.GetUpperBound(0) + 1;
            Int32 totalColumns = inMatrix.GetUpperBound(1) + 1;
            T3[,] lOut = new T3[totalRows, totalColumns];
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    lOut[rowIndex, colIndex] = inFunction(inMatrix[rowIndex, colIndex], inScalar);
                }
            }
            return lOut;
        }

        public static T3[,] Map<T1, T2, T3>(T1[,] inMatrix1, T2[,] inMatrix2, Func<T1, T2, T3> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix1.GetUpperBound(0) + 1;
            Int32 totalColumns = inMatrix1.GetUpperBound(1) + 1;
            T3[,] lOut = new T3[totalRows, totalColumns];
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    lOut[rowIndex, colIndex] = inFunction(inMatrix1[rowIndex, colIndex], inMatrix2[rowIndex, colIndex]);
                }
            }
            return lOut;
        }

        public static T2[,] Map<T1, T2>(T1[,] inMatrix, Func<T1, T2> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix.GetUpperBound(0) + 1;
            Int32 totalColumns = inMatrix.GetUpperBound(1) + 1;
            T2[,] lOut = new T2[totalRows, totalColumns];
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    lOut[rowIndex, colIndex] = inFunction(inMatrix[rowIndex, colIndex]);
                }
            }
            return lOut;
        }

        public static List<List<T2>> Map<T1, T2>(List<List<T1>> inMatrix, Func<T1, T2> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix.Count;
            Int32 totalColumns = inMatrix[0].Count;
            List<List<T2>> lOut = new List<List<T2>>();
            List<T2> outRow;
            List<T1> inRow;
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                outRow = new List<T2>();
                inRow = inMatrix[rowIndex];
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    outRow.Add(inFunction(inRow[colIndex]));
                }
                lOut.Add(outRow);
            }
            return lOut;
        }

        public static List<List<T3>> Map<T1, T2, T3>(List<List<T1>> inMatrix, T2 inScalar, Func<T1, T2, T3> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix.Count;
            Int32 totalColumns = inMatrix[0].Count;
            List<List<T3>> lOut = new List<List<T3>>();
            List<T3> outRow;
            List<T1> inRow;
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                outRow = new List<T3>();
                inRow = inMatrix[rowIndex];
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    outRow.Add(inFunction(inRow[colIndex],inScalar));
                }
                lOut.Add(outRow);
            }
            return lOut;
        }

        public static List<List<T3>> Map<T1, T2, T3>(List<List<T1>> inMatrix1, List<List<T2>> inMatrix2, Func<T1, T2, T3> inFunction)
        {
            Int32 rowIndex, colIndex;
            Int32 totalRows = inMatrix1.Count;
            Int32 totalColumns = inMatrix1[0].Count;
            List<List<T3>> lOut = new List<List<T3>>();
            List<T3> outRow;
            List<T1> inRow1;
            List<T2> inRow2;
            for (rowIndex = 0; rowIndex < totalRows; rowIndex++)
            {
                outRow = new List<T3>();
                inRow1 = inMatrix1[rowIndex];
                inRow2 = inMatrix2[rowIndex];
                for (colIndex = 0; colIndex < totalColumns; colIndex++)
                {
                    outRow.Add(inFunction(inRow1[colIndex], inRow2[colIndex]));
                }
                lOut.Add(outRow);
            }
            return lOut;
        }
    }
}
