﻿//Copyright Warren Harding 2011-2012.  All rights reserved.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Functional;
namespace Logic
{
    public class NBody
    {
        //interaction can be calculated one way, this produces a longer list
        public static Tuple<T, T>[] OrderedPairs<T>(T[] particles)
        {
            List<Tuple<T, T>> outPairs = new List<Tuple<T, T>>();
            Int32 L1, L2;
            Tuple<T, T> tuple;
            for (L1 = 0; L1 < particles.Length; L1++)
            {
                for (L2 = 0; L2 < particles.Length; L2++)
                {
                    if (L1 != L2)
                    {
                        tuple = new Tuple<T, T>(particles[L1], particles[L2]);
                        outPairs.Add(tuple);
                    }
                }
            }
            return outPairs.ToArray();
        }

        //interaction should be calculated both ways, this produces a shorter list
        public static Tuple<T,T>[] Pairs<T>(T[] particles)
        {
            List<Tuple<T, T>> outPairs = new List<Tuple<T, T>>();
            Int32 L1, L2;
            Tuple<T, T> tuple;
            for (L1 = 0; L1 < particles.Length-1; L1++)
            {
                for (L2 = L1 + 1; L2 < particles.Length; L2++)
                {
                    tuple = new Tuple<T, T>(particles[L1], particles[L2]);
                    outPairs.Add(tuple);
                }
            }
            return outPairs.ToArray();
        }

        //interaction should be calculated both ways
        public static TResult[] PairResults<TParticle, TResult>(TParticle[] particles, Func<TParticle, TParticle, TResult> action)
        {
            Tuple<TParticle, TParticle>[] tuples = Pairs(particles);
            TResult[] outResults = Maps.Map(tuples,(x)=>action(x.Item1 ,x.Item2));
            return outResults;
        }

    }
}
