﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Functional;
namespace Logic
{
    public class Set
    {
        //contained in hashset...

        public static Boolean IsSubSet<T>(T[] subSet, T[] superSet)
        {
            Boolean[] bools = Maps.Map(subSet, x => superSet.Contains(x));
            return Folds.AllTrue(bools);
        }

        public static T[] Union<T>(T[] set1, T[] set2)
        {
            T[] all = Concatenations.Concatenate(set1, set2);
            return all.Distinct().ToArray();
        }

        public static T[] Intersection<T>(T[] set1, T[] set2)
        {
            List<T> intersection = new List<T>();
            Int32 L;
            for (L = 0; L < set1.Length; L++)
            {
                if (set2.Contains(set1[L]))
                {
                    intersection.Add(set1[L]);
                }
            }
            return intersection.ToArray();
        }

        public static T[] Subtract<T>(T[] left, T[] right)
        {
            List<T> subtracted = new List<T>();
            Int32 L;
            for (L = 0; L < left.Length; L++)
            {
                if (left.Contains(right[L])==false)
                {
                    subtracted.Add(left[L]);
                }
            }
            return subtracted.ToArray();
        }

        public static T[] SymmetricDifference<T>(T[] set1, T[] set2)
        {
            List<T> intersection = new List<T>();
            Int32 L;
            for (L = 0; L < set1.Length; L++)
            {
                if (set2.Contains(set1[L])==false)
                {
                    intersection.Add(set1[L]);
                }
            }
            return intersection.ToArray();
        }

        public static Tuple<T, T>[] CartesianProduct<T>(T[] set1, T[] set2)
        {
            List<Tuple<T, T>> outPairs = new List<Tuple<T, T>>();
            Int32 L1, L2;
            Tuple<T, T> tuple;
            for (L1 = 0; L1 < set1.Length; L1++)
            {
                for (L2 = 0; L2 < set2.Length; L2++)
                {
                    tuple = new Tuple<T, T>(set1[L1], set2[L2]);
                    outPairs.Add(tuple);
                }
            }
            return outPairs.ToArray();
        }

        public static List<T[]> PowerSet<T>(T[] inSet)
        {
            Int32[] componentsPerDimension=Enumerable.Repeat(2,inSet.Length).ToArray();
            List<Int32[]> indexes = Logic.Combinatorics.NBaseIndexes(componentsPerDimension);
            Int32 setIndex, itemIndex;
            List<T> set;
            Int32[] setIndexes;
            List<T[]> lOut = new List<T[]>();
            for (setIndex = 0; setIndex < indexes.Count; setIndex++)
            {
                set = new List<T>();
                setIndexes = indexes[setIndex];
                for (itemIndex = 0; itemIndex < inSet.Length; itemIndex++)
                {
                    if (setIndexes[itemIndex] == 1)
                    {
                        set.Add(inSet[itemIndex]);
                    }
                }
                lOut.Add(set.ToArray());
            }
            return lOut;
        }

        
    }
}
