﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
namespace Tree
{
    public class TreeAndControl<T>
    {
        public TreeControl treeControl=new TreeControl();
        public TreeNode<T> tree;
        public TreeNode<T> clipBoard;
        public event Action ClipBoardChanged;
        public event Action Changing;
        public event Action Changed;
        public TreeAndControl()
        {
            treeControl.AddSiblingClicked += treeControl_AddSiblingClicked;
            treeControl.AddSubtreeClicked += treeControl_AddSubtreeClicked;
            treeControl.InsertClicked += treeControl_InsertClicked;
            treeControl.DeleteClicked += treeControl_DeleteClicked;
            treeControl.SelectedItemChangedClicked += treeControl_SelectedItemChangedClicked;
            treeControl.RenameClicked += treeControl_RenameClicked;
            treeControl.Cut += treeControl_Cut;
            treeControl.Copy += treeControl_Copy;
            treeControl.Paste += treeControl_Paste;
            tree = new TreeNode<T>();
            tree.name = "TrueRoot";
            TreeNode<T> rootChild = new TreeNode<T>();
            rootChild.name = "Root";
            tree.children.Add(rootChild);
            Bind();
        }

        public void Bind()
        {
            treeControl.TreeViewItem1.ItemsSource = tree.Children;
        }

        void treeControl_Cut()
        {
            Cut();
        }

        void treeControl_Paste()
        {
            Paste();
        }

        void treeControl_Copy()
        {
            Copy();
        }

        void treeControl_RenameClicked()
        {
            RenameNode();
        }

        void treeControl_SelectedItemChangedClicked()
        {
            Changed();
        }

        void treeControl_DeleteClicked()
        {
            DeleteNode();
        }

        void treeControl_InsertClicked()
        {
            InsertNode();
        }

        void treeControl_AddSubtreeClicked()
        {
            AddSubtree();
        }

        void treeControl_AddSiblingClicked()
        {
            AddSibling();
        }

        public TreeNode<T> SelectedNode
        {
            get
            {
                TreeView lTree = this.treeControl.TreeView1;
                if (lTree != null)
                {
                    try
                    {
                        return (TreeNode<T>)lTree.SelectedItem;
                    }
                    catch
                    {
                    }
                }
                return null;
            }
            set
            {
                TreeView treeView = this.treeControl.TreeView1;
                if (treeView != null)
                {
                    try
                    {
                        TreeNode<T> oldTree = (TreeNode<T>)treeView.SelectedItem;
                        TreeNode<T> newTree = value;
                        oldTree.parent.children.Insert(oldTree.Index, newTree);
                        oldTree.parent.children.Remove(oldTree);
                    }
                    catch
                    {
                    }
                }
            }
        }

        public void FireChanging()
        {
            if (Changing != null)
            {
                Changing();
            }
        }

        public void AddSibling()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            TreeNode<T> lTree = TreeHelper.AddSibling(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            Changed();
        }

        public void Cut()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            SerializableTreeNode<T> stn = SerializableTreeNode<T>.ToSerializable(SelectedNode);
            stn = Logic.Cloners.Clone(stn);
            TreeNode<T> tn = SerializableTreeNode<T>.ToObservable(stn);

            clipBoard = tn;
            SelectedNode.parent.children.Remove(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            if (ClipBoardChanged != null)
            {
                ClipBoardChanged();
            }
            Changed();
        }

        public void Copy()
        {
            FireChanging();
            SerializableTreeNode<T> stn = SerializableTreeNode<T>.ToSerializable(SelectedNode);
            stn = Logic.Cloners.Clone(stn);
            TreeNode<T> tn = SerializableTreeNode<T>.ToObservable(stn);

            clipBoard = tn;
            if (ClipBoardChanged != null)
            {
                ClipBoardChanged();
            }
            //clipBoard = Logic.Cloners.Clone(SelectedNode);
            Changed();
        }

        public void Paste()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            SerializableTreeNode<T> stn = SerializableTreeNode<T>.ToSerializable(clipBoard);
            stn = Logic.Cloners.Clone(stn);
            TreeNode<T> tn = SerializableTreeNode<T>.ToObservable(stn);

            SelectedNode = tn;
            this.treeControl.SelectedItemChangedEventIsOn = true;

            Changed();
        }


        public void AddSubtree()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            TreeNode<T> lTree = TreeHelper.AddSubtree(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            if (lTree == null)
            {
                MessageBox.Show("You have to select a node to add a subtree.");
                return;
            }
            Changed();
        }

        public void InsertNode()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            TreeNode<T> lTree = TreeHelper.InsertSibling(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            Changed();
        }

        public void DeleteNode()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            TreeHelper.DeleteNode(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            Changed();
        }

        public void RenameNode()
        {
            FireChanging();
            this.treeControl.SelectedItemChangedEventIsOn = false;
            TreeHelper.RenameNode(SelectedNode);
            this.treeControl.SelectedItemChangedEventIsOn = true;
            Changed();
        }


    }
}
